/**
Software SPI slave in port interrupt.

This code uses the P1 pins that are shared with the JTAG interface on
the F11x and F12x parts.

The port interrupts are used, no other P1 interrupts should be used.
Or at least their handling has to be added in this code.

The globals spi_data_out and spi_data_in are used as data registers.
When a byte is complete IRQ_SPI_rx is called, which must be an interrupt
function (does not need to be attached to a real interrupt vector, use
NOVECTOR instead). That function can read the received byte in spi_data_in
and it can set up the next outgoing character.

The data is copied to a separate shift register on byte start/stop, so that
spi_data_out and spi_data_in can be read or written at any time. But it's
recomened to read/write them in IRQ_SPI_rx only.

spi_data_out is set to 0xff after the data has been copied to the shift
register. This means that a 0xff is sent if no data is set up by the users
code.

Pins:
    P1.5 TMS  is used as low active chip select (/CS) a neg edge on
    this pin resets the SPI statemachine.

    P1.4 TCK  is used as SPI CLK input
    P1.6 TDI  is the data input SIMO
    P1.7 TDO  is the data output SOMI

NOTE: Unlike a real SPI device, the SPI pins are currently not switched
      to 3-state (input) when the chip select (/CS) is high.

NOTE: Due to the nature of a software implementation, it can not handle
      SPI clocks higher than a few kHz, depending on the CPU speed.
      It worked fine up to 43kHz with the DCO/MCLK at full speed (5 MHz).


(C) 2004 Chris Liechti <cliechti@gmx.net>
this is distributed under a free software license, see license.txt

http://mspgcc.sf.net
*/

#include "hardware.h"

;variables
.data
    .global spi_data_out
    .global spi_data_in
    .comm   spi_data_out,       1, 1            ;char var, not aligned
    .comm   spi_data_in,        1, 1            ;char var, not aligned

    .comm   spi_shift,          1, 1            ;char var, not aligned
    .comm   spi_state,          2, 2            ;short var, aligned
; use regs for faster code. must be used with "-ffixed-regs=..." !
//~ #define spi_shift r6
//~ #define spi_state r7

.text

/*
This one initializes the P1 settings used for the software SPI.
it also sets up the internal variables.
*/

.global spi_init
spi_init:
                mov.b   #0xff, spi_data_out
                mov.b   #0xff, spi_data_in
                clr     spi_state               ; reset state
                bis.b   #TDO, &P1OUT            ; set output
                mov.b   #(TCK|TMS), &P1IES      ; switch to neg edge detect
                mov.b   #(TCK|TMS), &P1IE       ; enable interrupts on CLK and CS
                ret


/*
Port interrupt handler. Shift one byte out, read one byte in, in SPI
slave fashion. The user must supply an interrupt function named IRQ_SPI_rx
which is called when a byte is complete.

It's implemented with a jump table and as fast as possible, however it's
nowhere as fast as a hardware SPI (see note in the comment above)...
*/
interrupt (PORT1_VECTOR)
.LSPI_shift:
                bit.b   #TCK, &P1IFG            ; clock edge?
                jz      .LSPI_CS                ; no -> jump
                bic.b   #TCK, &P1IFG            ; clear flag
                xor.b   #TCK, &P1IES            ; switch to neg/pos edge detect
                add     spi_state, r0           ; jump table
                jmp     .LSPI_bit_low_first     ; Do7
                jmp     .LSPI_bit_high          ; Di7
                jmp     .LSPI_bit_low           ; Do6
                jmp     .LSPI_bit_high          ; Di6
                jmp     .LSPI_bit_low           ; Do5
                jmp     .LSPI_bit_high          ; Di5
                jmp     .LSPI_bit_low           ; Do4
                jmp     .LSPI_bit_high          ; Di4
                jmp     .LSPI_bit_low           ; Do3
                jmp     .LSPI_bit_high          ; Di3
                jmp     .LSPI_bit_low           ; Do2
                jmp     .LSPI_bit_high          ; Di2
                jmp     .LSPI_bit_low           ; Do1
                jmp     .LSPI_bit_high          ; Di1
                jmp     .LSPI_bit_low           ; Do0
                //jmp   .LSPI_bit_high_last     ; Di0  code follows directly
.LSPI_bit_high_last:
                bit.b   #TDI, &P1IN             ; sample input
                rlc.b   spi_shift               ; and store it in the answer byte
                clr     spi_state               ; reset state
                mov.b   spi_shift, spi_data_in  ; copy incomming data
                eint
                br      #IRQ_SPI_rx             ; call user interrupt function on complete character

.LSPI_bit_high:
                bit.b   #TDI, &P1IN             ; sample input
                rlc.b   spi_shift               ; and store it in the answer byte
                incd    spi_state
                reti

.LSPI_bit_low_first:
                mov.b   spi_data_out, spi_shift ; copy outgoing data
                mov.b   #0xff, spi_data_out     ; set to idle so that the data is not repeated
.LSPI_bit_low:
                incd    spi_state
                tst.b   spi_shift               ; check msb
                jn      .Ltx_one
.Ltx_zero:      bic.b   #TDO, &P1OUT
                reti
.Ltx_one:       bis.b   #TDO, &P1OUT
                reti


.LSPI_CS:
                bit.b   #TMS, &P1IFG            ; was /CS activated?
                jz      .L_reti                 ; no -> jump
                bic.b   #TMS, &P1IFG            ; clear flag
                call    #spi_init               ; init interface on CS->LOW
.L_reti:        reti


