#!/usr/bin/env python
# Demonstration GUI for the SPI protocol implementation.
# The application protocol user over SPI is described in the main.c
# source file.
#
# (C) 2004 Chris Liechti <cliechti@gmx.net>
# this is distributed under a free software license, see license.txt
#
# http://mspgcc.sf.net
#
# Requires Python 2.3+ and the msp430 modules, wxPython 2.4+
#
# $Id: spicontroller.py,v 1.1 2004/07/16 01:19:55 cliechti Exp $

from msp430 import hilspi
from msp430 import HIL
import struct, binascii


#have to slow down SPI function so that the software implementation
#on the MSP430 can cope with it
def delay():
    #can't take HIL.DelayMSec(1) as the operating system rounds that up
    #to 10ms or longer. a value of 0 just makes a very short delay with
    #the function call overhead.
    for i in range(10): HIL.DelayMSec(0)        #hopfully work on every box
    #~ HIL.DelayMSec(0); HIL.DelayMSec(0)       #still works on a P4/2.4GHz
    
hilspi._delay = delay   #patch the delay function with the slower version

#Protocol control characters
SPI_X_IDLE      = '\xff'
SPI_X_DONE      = '\xfe'
SPI_X_BUSY      = '\xfd'
SPI_X_NACK      = '\xf1'
SPI_X_ACK       = '\xf0'

def command(cmd, arg=None):
    """This function send a command to the SPI controller and returns
    the received answer. An IOError is raised when the slave does not
    answer correctly or at all.
    
    The /CS line is activated while the command is active and
    deactivated afterwards.
    
    arg can be a string, which is sent as argument to the command.
    A binary string with the commands answer is returned. If the
    command was rejected by the slave, and IOError is raised.
    """
    HIL.TMS(0)  #enable /CS
    HIL.DelayMSec(1)
    try:
        #send command code
        hilspi.shift(chr(cmd))
        #optionaly, send argument (string)
        if arg is not None:
            if hilspi.shift(arg) != SPI_X_BUSY*len(arg):
                raise IOError("slave not ready while sending arguments")
        #finish command
        byte = hilspi.shift(SPI_X_DONE)
        if byte != SPI_X_BUSY:
            raise IOError("slave not ready while executing command (0x%02x)" % ord(byte))
        #now wait for answer
        answer = ''
        for tries in range(30000):
            byte = hilspi.shift(SPI_X_IDLE)
            if byte < '\xf0':           #data bytes
                answer += byte
            elif byte == SPI_X_ACK:
                break
            elif byte == SPI_X_NACK:
                raise IOError("command failed: %r" % answer)
            elif byte == SPI_X_BUSY:
                pass
        else:
            raise IOError("no answer within time")
        return answer
    finally:
        HIL.TMS(1)  #disable /CS

def reset():
    """Reset the hardware"""
    HIL.RST(0)
    HIL.DelayMSec(50)
    HIL.RST(1)
    HIL.DelayMSec(10)

def init():
    """init parallel port and reset the hardware"""
    hilspi.init()
    reset()

#commands implemented in the example
CMD_IDLE            = 0
CMD_SETLED          = 1
CMD_DELAY           = 2
CMD_SET_FREQ        = 3
CMD_SET_PWM         = 4

def setLed(state):
    """Set LED on or off"""
    return command(CMD_SETLED, state and '\x01' or '\x00')

def setFreq(f):
    """Set PWM base frequency, in timer units 0..65535"""
    #send freq as hex encoded 16 bit binary in little endian format
    return command(CMD_SET_FREQ, binascii.hexlify(struct.pack("<H", f)))

def setPwm(p1, p2):
    """Set both PWM duty cycles, in timer units 0..65535"""
    #send pwm settings as two hex encoded 16 bit binaries in LE format
    return command(CMD_SET_PWM, binascii.hexlify(struct.pack("<HH", p1, p2)))

#test application
if __name__ == '__main__':
    init()
    
    setLed(1)   #switch on LED while the sweep runs
    #~ #sweep in base frequency
    #~ for f in range(0, 1000, 100):
        #~ setFreq(f)
        #~ HIL.DelayMSec(100)
    
    #~ #sweep in pwm modulation
    setFreq(1000)
    for f in range(0, 1000, 100):
        setPwm(f, f)
        HIL.DelayMSec(100)
        
    setFreq(0)  #switch off freq output
    setLed(0)   #switch off LED
