#include <io.h>
;******************************************************************************
;   MSP-FET430x110 Demo - Software Output 6-bit R2R DAC
;
;   Description: Implement a 6-bit DAC using 6 P2.x outputs and an R2R
;   ladder.  A sine wave is generated using a 6-bit, 32-step sine look-up
;   table Sin_Tab.  R5 is used as a pointer into Sin_Tab.  Any MSP430 
;   outputs can be used for the DAC with ther bit-resolutions and
;   frequencies possible.  ACLK as an input to CCR2 is used to trigger an
;   interrupt at exacty 32768Hz.  The ISR updates the DAC using
;   Sin_Tab.  The frequency of the sine wave in this example is ISR 
;   frequency/steps = 32768/32 = 1024Hz. 
;   ACLK = LFXT1 = 32768, MCLK = SMCLK = default DCO ~ 800k 
;   //*An external watch crystal on XIN XOUT is required for ACLK*//	  
;
;
;                MSP430F1121
;             -----------------
;         /|\|              XIN|-  
;          | |                 | 32kHz
;          --|RST          XOUT|-
;            |             P2.5|--20k--+---> 1024Hz sine wave
;            |                 |      10k
;            |             P2.4|--20k--+
;            |                 |      10k
;            |             P2.3|--20k--+
;            |                 |      10k
;            |             P2.2|--20k--+
;            |                 |      10k
;            |             P2.1|--20k--+
;            |                 |      10k
;            |             P2.0|--20k--+
;            |                 |      20k
;                                      |
;                                      V
;   M.Buccini
;   Texas Instruments, Inc
;   January 2002
;   Adapted for mspgcc by Steve Underwood <steveu@coppice.org>
;******************************************************************************
            .section    .text
RESET:      mov.w   #0x300,R1               ; Initialize  stackpointer
StopWDT:    mov.w   #WDTPW+WDTHOLD,&WDTCTL  ; Stop watchdog timer
SetupBC:    bis.b   #RSEL2+RSEL0,&BCSCTL1   ; RSEL=5
SetupP2:    mov.b   #0x3F,&P2DIR            ; P2.0 - 5 output direction
            clr.w   R5                      ; Clear pointer
SetupTA:    mov.w   #TASSEL1+TACLR,&TACTL    ; SMCLK, Clear TAR
Setup_CC2:  mov.w   #CCIS0+CM0+CAP+CCIE,&CCTL2   ; CCR2,CAP on CCxB,ACLK,int									
            bis.w   #MC1,&TACTL             ; Continous Mode
            eint                            ; Enable interrupts
                                            ;
Mainloop:   jmp     Mainloop                ; Do Nothing
                                            ;
;------------------------------------------------------------------------------ 
TAX_ISR:    ;Common ISR for CCR1-4 and overflow
;------------------------------------------------------------------------------ 
            add.w   &TAIV,R0                ; Add TA interrupt offset to PC
            reti                            ; CCR0 - no source
            reti                            ; CCR1
            jmp     CCR2_ISR                ; CCR2
;            reti                            ; CCR3
;            reti                            ; CCR4
;TA_over:    reti                            ; Timer_A overflow
                                            ;
CCR2_ISR:   mov.b   Sin_tab(R5),&P2OUT      ; Move sine value to P2
            inc.b   R5                      ; Inc pointer
            and.b   #0x1F,R5                ; R5 = 0 - 31 only (32 bytes)             
            reti                            ;
;------------------------------------------------------------------------------ 
Sin_tab:    ;Sine 6-bit Lookup table with 32 steps
;------------------------------------------------------------------------------ 
            .byte    32
            .byte    38
            .byte    44
            .byte    49
            .byte    54
            .byte    58
            .byte    61
            .byte    62
            .byte    63
            .byte    62
            .byte    61
            .byte    58
            .byte    54
            .byte    49
            .byte    44
            .byte    38
            .byte    32
            .byte    25
            .byte    19
            .byte    14
            .byte    9
            .byte    5
            .byte    2
            .byte    1
            .byte    0
            .byte    1
            .byte    2
            .byte    5
            .byte    9
            .byte    14
            .byte    19
            .byte    25

;------------------------------------------------------------------------------ 
;           Interrupt Vectors Used MSP430x11x1          
;------------------------------------------------------------------------------ 
            .section    .vectors
            .org    0x10,0xFF               ; Timer_AX Vector
            .word   TAX_ISR                 ; 
            .org    0x1E,0xFF               ; MSP430 RESET Vector
            .word   RESET                   ; 
