#include <io.h>
;******************************************************************************
;   MSP-FET430P440 Demo - ADC12, Converison of the Temperature Diode
;
;   Description: This program will show how to use the integrated temperature
;                diode to measure temperature.
;
;   This example shows how to use the intergrated temperature diode to measure
;   temperature. When the temperature diode channel (A10) is selected for 
;   conversion, the internal reference is automatically turned on as the source 
;   for the diode. Note however, that it is NOT automatically selected for the 
;   conversion.  Any available reference can be used for the conversion.  In 
;   this example, a single conversion is performed of the temperature diode.  
;   The temperature is then determined from a look-up table, based on the A/D 
;   conversion value. The A/D conversion results are stored in ADC12MEM0. The 
;   temperature is shown in R5 in BCD format, degress F.  Test by setting and 
;   running to a break point at "jmp Mainloop."  To view the temperature, open 
;   a register window in C-Spy and view the contents of R5.
;
;   Note: This example does not perform a calibration on the temperature diode
;   A calibration of the temperature diode may be necessary in an application.
;   See the device datasheet for the temperature diode specification. 
;
;   M.Mitchell
;   Texas Instruments, Inc
;   January, 2002
;   Adapted for mspgcc by Steve Underwood <steveu@coppice.org>
;******************************************************************************
            .section    .text
main:
RESET:      mov     #0xA00,R1               ; Initialize stackpointer
StopWDT:    mov     #WDTPW+WDTHOLD,&WDTCTL  ; Stop watchdog
                                            ;
SetupADC12: mov     #ADC12ON+REFON+REF2_5V+SHT0_6,&ADC12CTL0 
                                            ; Turn on ADC12, 2.5Vref, set SHT0
                                            ; for longer sampling required by
                                            ; temp diode.
            mov     #SHP,&ADC12CTL1         ; Use sampling timer    
            mov.b   #INCH_10+SREF_1,&ADC12MCTL0  ; Select channel A10, Vref+
            bis     #BIT0,&ADC12IE          ; Enable ADC12IFG.0 for ADC12MEM0            
                                            ;
            mov     #0x3600,R15             ; Delay needed for ref start-up
L1:         dec     R15                     ; See datasheet for details
            jnz     L1                      ;
                                            ;
            bis     #ENC,&ADC12CTL0         ; Enable conversions
            eint                            ; Enable interrupts         
                                            ;
Mainloop:   bis     #ADC12SC,&ADC12CTL0     ; Start conversions
            bis     #CPUOFF,R2              ; Wait for conversion completion
                                            ;
DispTemp:   mov     #0x00,R15               ; Clear table pointer
            mov     #0x32,R5                ; Load R5 with min. temp
            jmp     First_cmp               ; Compare without incrementing
CMPloop:    incd    R15                     ; Point to next value in table
            dadd    #1,R5                   ; Decimally increment R5
First_cmp:  cmp     Temp_Tab(R15),R6        ; Compare A/D result
            jge     CMPloop                 ; Jump to increment and compare
            jmp     Mainloop                ; SET BERAKPOINT HERE.
                                            ; Done comparing. Temp in R5
                                            ;
;------------------------------------------------------------------------------ 
;           Temperature Table       
;           These values assume nominal values for the temp diode and
;           temperature coefficient.  They also assume the 2.5V internal
;           reference is used for the conversion.  
;------------------------------------------------------------------------------
Temp_Tab:   .word   0x64F                   ; 32F
            .word   0x652
            .word   0x656
            .word   0x659                   ;35F
            .word   0x65C
            .word   0x65F
            .word   0x662
            .word   0x666
            .word   0x669                   ;40F
            .word   0x66C
            .word   0x66F
            .word   0x673
            .word   0x676
            .word   0x679                   ;45F
            .word   0x67C
            .word   0x680
            .word   0x683
            .word   0x686
            .word   0x689                   ;50F
            .word   0x68C
            .word   0x690
            .word   0x693
            .word   0x696
            .word   0x699                   ;55F
            .word   0x69D
            .word   0x6A0
            .word   0x6A3
            .word   0x6A6
            .word   0x6AA                   ;60F
            .word   0x6AD
            .word   0x6B0
            .word   0x6B3
            .word   0x6B6
            .word   0x6BA                   ;65F
            .word   0x6BD
            .word   0x6C0
            .word   0x6C3
            .word   0x6C7
            .word   0x6CA                   ;70F
            .word   0x6CD          
            .word   0x6D0
            .word   0x6D4
            .word   0x6D7
            .word   0x6DA                   ;75F
            .word   0x6DD
            .word   0x6E0
            .word   0x6E4
            .word   0x6E7
            .word   0x6EA                   ;80F
            .word   0x6ED
            .word   0x6F1
            .word   0x6F4
            .word   0x6F7
            .word   0x6FA                   ;85F
            .word   0x6FE
            .word   0x701
            .word   0x704
            .word   0x707
            .word   0x70B                   ;90F
            .word   0x70E
            .word   0x711
            .word   0x714
            .word   0x717
            .word   0x71B                   ;95F
            .word   0x71E
            .word   0x721
            .word   0x724
            .word   0x728
            .word   0x72B                   ;100F
            .word   0xFFF                   ;Too high            
;------------------------------------------------------------------------------ 
ADC12ISR:   ;Interrupt Service Routine for ADC12
;------------------------------------------------------------------------------
            mov     &ADC12MEM0,R6           ; Move result, IFG is reset
            bic     #CPUOFF,0(R1)           ; Return active
            reti                            ;
                                            ;
;------------------------------------------------------------------------------ 
;           Interrupt Vectors         
;------------------------------------------------------------------------------
            .section    .vectors
            .org    0x0E,0xFF               ; ADC12 Interrupt Vector
            .word   ADC12ISR                ;
            .org    0x1E,0xFF               ; MSP430 RESET Vector
            .word   RESET                   ;
